/**********************************************************************************************
	Red Rapids
	797 N. Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	Description: Driver functions that are common to all products.

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

***********************************************************************************************/

#ifndef _RRADAPTER_H_
#define _RRADAPTER_H_

#if defined(__cplusplus)
    extern "C" {
#endif

// Define the maximum number of devices supported by the driver.
#define MAX_NUM_DEVICES		4

// Define error codes returned by the driver.
#define OK					0
#define DRIVER_FAIL			-400
#define OPEN_FAIL			-410
	#define KERNEL_HANDLE		(OPEN_FAIL - 1)
	#define PCI_SCAN			(OPEN_FAIL - 2)
	#define NO_DEV_FOUND		(OPEN_FAIL - 3)
	#define DEV_OUT_OF_RANGE	(OPEN_FAIL - 4)
	#define PCI_INFO			(OPEN_FAIL - 5)
	#define HANDLE_ATTACH		(OPEN_FAIL - 6)
	#define MEM_ALLOCATE		(OPEN_FAIL - 7)
	#define INT_ENABLE			(OPEN_FAIL - 8)
#define CLOSE_FAIL			-420
#define RDWR_FAIL			-430
	#define RD_FAIL				(RDWR_FAIL - 1)
	#define WR_FAIL				(RDWR_FAIL - 2)
#define DMA_FAIL			-440
	#define DMA_ALLOCATE		(DMA_FAIL - 1)
	#define DMA_FREE			(DMA_FAIL - 2)
#define SLEEP_FAIL			-450

typedef struct {
	unsigned SubsystemID;			// Unique identifer for the bus interface characteristics.
	unsigned long Bus;				// PCI bus number reported by the driver (zero based).
	unsigned long Slot;				// PCI slot number reported by the driver (zero based).
	unsigned long Function;			// PCI function number reported by the driver (zero based).
} s_DEVMap;

// Structure to store buffer information retrieved from Red Rapids DMA character devices (/dev/rrdma#).
typedef struct {
	unsigned PageNumber;			// Number of the active DMA page starting from zero.
	unsigned PageCount;				// Number of DMA pages available in the buffer.
	unsigned PagesPerMark;			// Number of DMA pages to transfer between completion marker (status) flags.
	unsigned BurstSizeB;			// Size (bytes) of each burst transaction.
	unsigned BurstCount;			// Number of burst transactions per DMA page.
	unsigned long long **Address;	// Virtual addresses assigned to each DMA page.
} s_DMABuf;

// Structure to store the Red Rapids device hardware handle.
typedef struct {
	unsigned DevNum;				// Device number assigned to the Red Rapids hardware (zero based).
	s_DEVMap DEV;					// Device structure.
	s_DMABuf *DMA;					// DMA config for all DMA buffers.
	void *User;						// Undefined pointer available to user applications.
} s_Handle;

#ifdef WIN32
	#define EXPORTINT __declspec(dllexport) int
#else
	#define EXPORTINT int
#endif

EXPORTINT DriverName(const char* Name);
EXPORTINT HandleInit(s_Handle *p_Adapter);

EXPORTINT BusOpen(s_Handle *p_Adapter);
EXPORTINT BusClose(s_Handle *p_Adapter);
EXPORTINT BusScan(void);

EXPORTINT Read8(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned char *pData);
EXPORTINT Read32(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned *pData);
EXPORTINT Read64(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned long long *pData);

EXPORTINT Write8(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned char Data);
EXPORTINT Write32(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned Data);
EXPORTINT Write64(s_Handle *p_Adapter, unsigned long BAR, unsigned long Offset, unsigned long long Data);

EXPORTINT DMABufAllocate(s_DMABuf *DMA, unsigned DevNum, unsigned long long *PhysicalAddress);
EXPORTINT DMABufFree(s_DMABuf *DMA, unsigned DevNum);

EXPORTINT MicrosecondTimer(unsigned long uSeconds);

#ifdef __cplusplus
}
#endif

#endif

