/**********************************************  Interrupt Service Routine (ISR) **********************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following ISR will execute each time the driver detects an interrupt produced by the device.
	Note:  This example assumes that channel interrupts are enabled but fault interrupts are disabled.

	Copyright (c) 2019  Red Rapids, Inc.

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
	files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
	modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
	is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
	IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

**********************************************  Interrupt Service Routine (ISR) **********************************************/

#ifdef WIN32
	#define EXPORTISR __declspec(dllexport) void
#else
	#define EXPORTISR void
#endif

#include <stdlib.h>

#include "RRadapter.h"				// Adpater device driver common to all Red Rapids PCI Express products.
#include "sigfpga.h"				// SigFPGA application.
#include "global_functions.h"		// Functions common to all Signal Stream hardware and channel types.
#include "utility_functions.h"		// Functions specific to this application example.

EXPORTISR RRadapter_ISR(s_Handle *p_Adapter)
{
	// Process all channels for any status changes.
	ProcessChannels(p_Adapter);

	// The global interrupt mask was set to zero by the driver to suppress further interrupts, so it has to be enabled before exiting the ISR.
	InterruptMask(p_Adapter, 1);
}
